@@ -17,22 +17,20 @@
 use std::io;
 use std::rc::Rc;
 
-use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
+use fallible_iterator::{FallibleIterator, IntoFallibleIterator};
 
-use super::super::{Error, ErrorKind, Result};
-use super::super::common::{KeyPairRef, IntoVerified, Refresh,
-                           ResourceQuery, ResourceIterator};
+use super::super::common::{IntoVerified, KeyPairRef, Refresh, ResourceIterator, ResourceQuery};
 use super::super::session::Session;
 use super::super::utils::Query;
+use super::super::{Error, ErrorKind, Result};
 use super::base::V2API;
 use super::protocol;
 
-
 /// Structure representing a key pair.
 #[derive(Clone, Debug)]
 pub struct KeyPair {
     session: Rc<Session>,
-    inner: protocol::KeyPair
+    inner: protocol::KeyPair,
 }
 
 /// A query to server list.
@@ -47,20 +45,14 @@ pub struct KeyPairQuery {
 #[derive(Clone, Debug)]
 pub struct NewKeyPair {
     session: Rc<Session>,
-    name: String,
-    public_key: Option<String>,
+    inner: protocol::KeyPairCreate,
 }
 
-
 impl KeyPair {
     /// Load a KeyPair object.
-    pub(crate) fn new<Id: AsRef<str>>(session: Rc<Session>, id: Id)
-            -> Result<KeyPair> {
+    pub(crate) fn new<Id: AsRef<str>>(session: Rc<Session>, id: Id) -> Result<KeyPair> {
         let inner = session.get_keypair(id)?;
-        Ok(KeyPair {
-            session: session,
-            inner: inner
-        })
+        Ok(KeyPair { session, inner })
     }
 
     /// Delete the key pair.
@@ -95,7 +87,7 @@ impl Refresh for KeyPair {
 impl KeyPairQuery {
     pub(crate) fn new(session: Rc<Session>) -> KeyPairQuery {
         KeyPairQuery {
-            session: session,
+            session,
             query: Query::new(),
             can_paginate: true,
         }
@@ -155,12 +147,10 @@ impl KeyPairQuery {
 
 impl NewKeyPair {
     /// Start creating a key pair.
-    pub(crate) fn new(session: Rc<Session>, name: String)
-            -> NewKeyPair {
+    pub(crate) fn new(session: Rc<Session>, name: String) -> NewKeyPair {
         NewKeyPair {
-            session: session,
-            name: name,
-            public_key: None,
+            session,
+            inner: protocol::KeyPairCreate::new(name),
         }
     }
 
@@ -168,40 +158,84 @@ impl NewKeyPair {
     ///
     /// This call fails immediately if no public_key is provided.
     pub fn create(self) -> Result<KeyPair> {
-        let request = if let Some(public_key) = self.public_key {
-            protocol::KeyPairCreate {
-                key_type: None,  // TODO
-                name: self.name,
-                public_key: public_key
-            }
-        } else {
-            return Err(Error::new(ErrorKind::InvalidInput,
-                                  "Public key contents is required"));
+        if self.inner.public_key.is_none() {
+            return Err(Error::new(
+                ErrorKind::InvalidInput,
+                "Public key contents is required",
+            ));
         };
 
-        let keypair = self.session.create_keypair(request)?;
+        let keypair = self.session.create_keypair(self.inner)?;
         Ok(KeyPair {
             session: self.session,
-            inner: keypair
+            inner: keypair,
         })
     }
 
+    /// Create a key pair, generating its public key.
+    ///
+    /// Returns a new key pair and its private key.
+    pub fn generate(mut self) -> Result<(KeyPair, String)> {
+        self.inner.public_key = None;
+
+        let mut keypair = self.session.create_keypair(self.inner)?;
+        if let Some(private_key) = keypair.private_key.take() {
+            let result = KeyPair {
+                session: self.session,
+                inner: keypair,
+            };
+
+            Ok((result, private_key))
+        } else {
+            Err(Error::new(
+                ErrorKind::InvalidResponse,
+                "Missing private key in the response",
+            ))
+        }
+    }
+
+    creation_inner_field! {
+        #[doc = "Set type of the key pair."]
+        set_key_type, with_key_type -> key_type: optional protocol::KeyPairType
+    }
+
+    creation_inner_field! {
+        #[doc = "Set name of the key pair."]
+        set_name, with_name -> name: String
+    }
+
+    creation_inner_field! {
+        #[doc = "Set name of the key pair."]
+        set_public_key, with_public_key -> public_key: optional String
+    }
+
     /// Add public key from a reader.
-    pub fn from_reader<R>(self, reader: &mut R) -> io::Result<NewKeyPair> where R: io::Read {
+    #[deprecated(since = "0.2.2", note = "Use with_public_key")]
+    pub fn from_reader<R>(self, reader: &mut R) -> io::Result<NewKeyPair>
+    where
+        R: io::Read,
+    {
         let mut s = String::new();
         let _ = reader.read_to_string(&mut s)?;
-        Ok(self.from_string(s))
+        Ok(self.with_public_key(s))
     }
 
     /// Add public key from a string.
-    pub fn from_string<S>(mut self, public_key: S) -> NewKeyPair where S: Into<String> {
-        self.set_string(public_key);
-        self
+    #[deprecated(since = "0.2.2", note = "Use with_public_key")]
+    pub fn from_string<S>(self, public_key: S) -> NewKeyPair
+    where
+        S: Into<String>,
+    {
+        self.with_public_key(public_key)
     }
 
     /// Add public key from a string.
-    pub fn set_string<S>(&mut self, public_key: S) where S: Into<String> {
-        self.public_key = Some(public_key.into());
+    #[deprecated(since = "0.2.2", note = "Use set_public_key")]
+    pub fn set_string<S>(&mut self, public_key: S)
+    where
+        S: Into<String>,
+    {
+        self.set_public_key(public_key);
     }
 }
 
@@ -222,13 +256,17 @@ impl ResourceQuery for KeyPairQuery {
         resource.name().clone()
     }
 
-    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>)
-            -> Result<Vec<Self::Item>> {
+    fn fetch_chunk(&self, limit: Option<usize>, marker: Option<String>) -> Result<Vec<Self::Item>> {
         let query = self.query.with_marker_and_limit(limit, marker);
-        Ok(self.session.list_keypairs(&query)?.into_iter().map(|item| KeyPair {
-            session: self.session.clone(),
-            inner: item
-        }).collect())
+        Ok(self
+            .session
+            .list_keypairs(&query)?
+            .into_iter()
+            .map(|item| KeyPair {
+                session: self.session.clone(),
+                inner: item,
+            })
+            .collect())
     }
 }
 
