@@ -102,6 +102,7 @@ extern crate waiter;
 macro_rules! transparent_property {
     ($(#[$attr:meta])* $name:ident: ref $type:ty) => (
         $(#[$attr])*
+        #[inline]
         pub fn $name(&self) -> &$type {
             &self.inner.$name
         }
@@ -109,6 +110,7 @@ macro_rules! transparent_property {
 
     ($(#[$attr:meta])* $name:ident: $type:ty) => (
         $(#[$attr])*
+        #[inline]
         pub fn $name(&self) -> $type {
             self.inner.$name
         }
@@ -132,6 +134,7 @@ macro_rules! query_filter {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<T: Into<String>>(mut self, value: T) -> Self {
             self.$set_func(value);
             self
@@ -140,21 +143,22 @@ macro_rules! query_filter {
 
     ($(#[$attr:meta])* $func:ident -> $name:ident: $type:ty) => (
         $(#[$attr])*
-        pub fn $func(mut self, value: $type) -> Self {
-            self.query.push(stringify!($name), value);
+        pub fn $func<T: Into<$type>>(mut self, value: T) -> Self {
+            self.query.push(stringify!($name), value.into());
             self
         }
     );
 
     ($(#[$attr:meta])* $set_func:ident, $with_func:ident -> $name:ident: $type:ty) => (
         $(#[$attr])*
-        pub fn $set_func(&mut self, value: $type)  {
-            self.query.push(stringify!($name), value);
+        pub fn $set_func<T: Into<$type>>(&mut self, value: T)  {
+            self.query.push(stringify!($name), value.into());
         }
 
         $(#[$attr])*
-        pub fn $with_func(mut self, value: $type) -> Self {
-            self.$set_func(value);
+        #[inline]
+        pub fn $with_func<T: Into<$type>>(mut self, value: T) -> Self {
+            self.$set_func(value.into());
             self
         }
     );
@@ -165,11 +169,13 @@ macro_rules! creation_inner_field {
 
     ($(#[$attr:meta])* $set_func:ident, $with_func:ident -> $name:ident) => (
         $(#[$attr])*
+        #[inline]
         pub fn $set_func<S: Into<String>>(&mut self, value: S)  {
             self.inner.$name = value.into();
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<S: Into<String>>(mut self, value: S) -> Self {
             self.$set_func(value);
             self
@@ -178,11 +184,13 @@ macro_rules! creation_inner_field {
 
     ($(#[$attr:meta])* $set_func:ident, $with_func:ident -> $name:ident: $type:ty) => (
         $(#[$attr])*
+        #[inline]
         pub fn $set_func(&mut self, value: $type)  {
             self.inner.$name = value;
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$set_func(value);
             self
@@ -191,11 +199,13 @@ macro_rules! creation_inner_field {
 
     ($(#[$attr:meta])* $set_func:ident, $with_func:ident -> $name:ident: optional String) => (
         $(#[$attr])*
+        #[inline]
         pub fn $set_func<S: Into<String>>(&mut self, value: S)  {
             self.inner.$name = Some(value.into());
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<S: Into<String>>(mut self, value: S) -> Self {
             self.$set_func(value);
             self
@@ -204,11 +214,13 @@ macro_rules! creation_inner_field {
 
     ($(#[$attr:meta])* $set_func:ident, $with_func:ident -> $name:ident: optional $type:ty) => (
         $(#[$attr])*
+        #[inline]
         pub fn $set_func(&mut self, value: $type)  {
             self.inner.$name = Some(value);
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$set_func(value);
             self
@@ -227,11 +239,13 @@ macro_rules! creation_inner_vec {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $name(&mut self) -> &mut Vec<String> {
             &mut self.inner.$name
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<S: Into<String>>(mut self, value: S) -> Self {
             self.$add_func(value);
             self
@@ -245,11 +259,13 @@ macro_rules! creation_inner_vec {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $name(&mut self) -> &mut Vec<$type> {
             &mut self.inner.$name
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$add_func(value);
             self
@@ -269,6 +285,7 @@ macro_rules! update_field {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<S: Into<String>>(mut self, value: S) -> Self {
             self.$set_func(value);
             self
@@ -284,6 +301,7 @@ macro_rules! update_field {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$set_func(value);
             self
@@ -299,6 +317,7 @@ macro_rules! update_field {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func<S: Into<String>>(mut self, value: S) -> Self {
             self.$set_func(value);
             self
@@ -314,6 +333,7 @@ macro_rules! update_field {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$set_func(value);
             self
@@ -341,6 +361,7 @@ macro_rules! update_field_mut {
         }
 
         $(#[$attr])*
+        #[inline]
         pub fn $with_func(mut self, value: $type) -> Self {
             self.$set_func(value);
             self
@@ -374,12 +395,12 @@ macro_rules! save_fields {
 #[allow(unused_macros)]
 macro_rules! protocol_enum {
     {$(#[$attr:meta])* enum $name:ident: $carrier:ty {
-        $($item:ident = $val:expr),+
+        $($(#[$iattr:meta])* $item:ident = $val:expr),+
     }} => (
         $(#[$attr])*
         #[derive(Debug, Clone, Copy, PartialEq, Eq)]
         pub enum $name {
-            $($item),+,
+            $($(#[$iattr])* $item),+,
             #[doc(hidden)]
             __Nonexhaustive,
         }
@@ -422,12 +443,12 @@ macro_rules! protocol_enum {
     );
 
     {$(#[$attr:meta])* enum $name:ident {
-        $($item:ident = $val:expr),+
+        $($(#[$iattr:meta])* $item:ident = $val:expr),+
     }} => (
         $(#[$attr])*
         #[derive(Debug, Clone, Copy, PartialEq, Eq)]
         pub enum $name {
-            $($item),+,
+            $($(#[$iattr])* $item),+,
             #[doc(hidden)]
             __Nonexhaustive,
         }
@@ -463,7 +484,7 @@ macro_rules! protocol_enum {
         }
 
         impl ::serde::ser::Serialize for $name {
-            fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+            fn serialize<S>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error>
                     where S: ::serde::ser::Serializer {
                 serializer.serialize_str(self.as_ref())
             }
